//=============================================================================
// File: ignutmsg.h
// Author: Andre Dufour
//
// Class type: interface
// Description: interface for all gnutella messages.
//=============================================================================

#ifndef __IGNUTMSG_H__
#define __IGNUTMSG_H__

#include <stddef.h>

class GnutCoord;

class IGnutMsg
{
protected:
    // = FOUNDATION

    IGnutMsg(void);
    virtual ~IGnutMsg(void);

public:
    // = ACTION

    virtual void writeToPacket(void* aMem) const = 0;

    
    // = ACCESS

    // Get size of message in bytes
    virtual size_t getSize(void) const = 0;

    virtual const GnutCoord& getNeighbourCoordinates(void) const = 0;

    virtual double getNeighbourError(void) const = 0;

    virtual double getNeighbourTimeSent(void) const = 0;


private:
    // = FOUNDATION

    // Disable copy constructor and assignment operator
    IGnutMsg(const IGnutMsg&);
    IGnutMsg& operator=(const IGnutMsg&);

};
//=============================================================================
#endif /* __IGNUTMSG_H__ */

